package com.sun.xml.internal.ws.api.addressing;

import org.xml.sax.SAXException;
import org.xml.sax.ErrorHandler;
import org.xml.sax.ContentHandler;
import org.w3c.dom.Node;
import javax.xml.soap.SOAPHeader;
import javax.xml.transform.Transformer;
import javax.xml.soap.SOAPException;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.ByteArrayInputStream;
import javax.xml.stream.XMLOutputFactory;
import java.io.ByteArrayOutputStream;
import com.sun.xml.internal.ws.util.xml.XmlUtil;
import javax.xml.soap.SOAPMessage;
import javax.xml.stream.XMLStreamWriter;
import com.sun.istack.internal.Nullable;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamException;
import com.sun.istack.internal.NotNull;
import javax.xml.namespace.QName;
import com.sun.xml.internal.ws.message.AbstractHeaderImpl;

final class EPRHeader extends AbstractHeaderImpl
{
    private final String nsUri;
    private final String localName;
    private final WSEndpointReference epr;
    
    EPRHeader(QName tagName, WSEndpointReference epr) {
        this.nsUri = tagName.getNamespaceURI();
        this.localName = tagName.getLocalPart();
        this.epr = epr;
    }
    
    @NotNull
    public String getNamespaceURI() {
        return this.nsUri;
    }
    
    @NotNull
    public String getLocalPart() {
        return this.localName;
    }
    
    @Nullable
    public String getAttribute(@NotNull String nsUri, @NotNull String localName) {
        try {
            XMLStreamReader sr = this.epr.read("EndpointReference");
            while (sr.getEventType() != 1) {
                sr.next();
            }
            return sr.getAttributeValue(nsUri, localName);
        }
        catch (XMLStreamException e) {
            throw new AssertionError(e);
        }
    }
    
    public XMLStreamReader readHeader() throws XMLStreamException {
        return this.epr.read(this.localName);
    }
    
    public void writeTo(XMLStreamWriter w) throws XMLStreamException {
        this.epr.writeTo(this.localName, w);
    }
    
    public void writeTo(SOAPMessage saaj) throws SOAPException {
        try {
            Transformer t = XmlUtil.newTransformer();
            SOAPHeader header = saaj.getSOAPHeader();
            if (header == null) {
                header = saaj.getSOAPPart().getEnvelope().addHeader();
            }
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            XMLStreamWriter w = XMLOutputFactory.newFactory().createXMLStreamWriter(baos);
            this.epr.writeTo(this.localName, w);
            w.flush();
            ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
            DocumentBuilderFactory fac = DocumentBuilderFactory.newInstance();
            fac.setNamespaceAware(true);
            Node eprNode = fac.newDocumentBuilder().parse(bais).getDocumentElement();
            Node eprNodeToAdd = header.getOwnerDocument().importNode(eprNode, true);
            header.appendChild(eprNodeToAdd);
        }
        catch (Exception e) {
            throw new SOAPException(e);
        }
    }
    
    public void writeTo(ContentHandler contentHandler, ErrorHandler errorHandler) throws SAXException {
        this.epr.writeTo(this.localName, contentHandler, errorHandler, true);
    }
}
